///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2015 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Reflection;

using XLR8.CGLib;

using com.espertech.esper.client;
using com.espertech.esper.events.vaevent;
using com.espertech.esper.util;

namespace com.espertech.esper.events.bean
{
    /// <summary>
    /// Property getter using XLR8.CGLib's FastMethod instance.
    /// </summary>
    public sealed class CGLibPropertyMethodGetter 
        : BaseNativePropertyGetter
        , BeanEventPropertyGetter
    {
        private readonly FastMethod _fastMethod;
    
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="method">the underlying method</param>
        /// <param name="fastMethod">The fast method.</param>
        /// <param name="eventAdapterService">factory for event beans and event types</param>
        public CGLibPropertyMethodGetter(MethodInfo method, FastMethod fastMethod, EventAdapterService eventAdapterService)
            : base(eventAdapterService, fastMethod.ReturnType, TypeHelper.GetGenericReturnType(method, true))
        {
            _fastMethod = fastMethod;
        }

        public Object GetBeanProp(Object obj)
        {
            try
            {
            return _fastMethod.Invoke(obj);
            }
            catch (InvalidCastException e)
            {
                throw PropertyUtility.GetMismatchException(_fastMethod.Target, obj, e);
            }
            catch (PropertyAccessException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new PropertyAccessException(e);
            }
        }

        public bool IsBeanExistsProperty(Object obj)
        {
            return true; // Property exists as the property is not dynamic (unchecked)
        }
    
        public override Object Get(EventBean obj)
        {
            return GetBeanProp(obj.Underlying);
        }
    
        public override String ToString()
        {
            return "CGLibPropertyGetter " +
                    "fastMethod=" + _fastMethod;
        }
    
        public override bool IsExistsProperty(EventBean eventBean)
        {
            return true; // Property exists as the property is not dynamic (unchecked)
        }
    }
}
